/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include "project.h"

/*Funciton protorype for handling errors*/
void HandleError(void);

/*Logical Size of Em_EEPROM*/
#define LOGICAL_EEPROM_SIZE     15u
#define LOGICAL_EEPROM_START    0u

/*location of reset counter in Em_EEPROM*/
#define RESET_COUNT_LOCATION    13u
#define RESET_COUNT_SIZE        2u

/*ASCII "9"*/
#define ASCII_NINE              0x39

/*ASCII "0"*/
#define ASCII_ZERO              0x30

/*ASCII "P"*/
#define ASCII_P                 0x50

/*Return status for EEPROM and UART*/ 
cy_en_em_eeprom_status_t eepromReturnValue;
cy_en_scb_uart_status_t uartReturnValue;

/* EEPROM storage in work flash, this is defined in Em_EEPROM.c*/
extern const uint8_t Em_EEPROM_em_EepromStorage[];

/*RAM arrays for holding EEPROM read and write data*/
uint8_t eepromArray[15];
uint8_t array[15] = { 0x50, 0x6F, 0x77, 0x65, 0x72, 0x20, 0x43, 0x79, 0x63, 0x6C, 0x65, 0x23, 0x20, 0x30, 0x30};
                               /* P     o     w     e     r           C     y     c     l     e     #           0*/


int main(void)
{
    __enable_irq(); /* Enable global interrupts. */
    /* Enable CM4.  CY_CORTEX_M4_APPL_ADDR must be updated if CM4 memory layout is changed. */
    Cy_SysEnableCM4(CY_CORTEX_M4_APPL_ADDR); 

    /* Place your initialization/startup code here (e.g. MyInst_Start()) */
     /* Set Start Address to work flash */
    Em_EEPROM_config.userFlashStartAddr = (uint32_t)Em_EEPROM_em_EepromStorage;
    
    /*Initalize EEPROM, config and context structures defined in Em_EEPROM.c/.h*/
    eepromReturnValue = Cy_Em_EEPROM_Init(&Em_EEPROM_config, &Em_EEPROM_context);
    if(eepromReturnValue != CY_EM_EEPROM_SUCCESS)
    {
        HandleError();
    }
    
    /*Initalize UART for tranmitting EEPROM contents*/
    uartReturnValue = Cy_SCB_UART_Init(UART_HW, &UART_config, &UART_context);
    if(uartReturnValue != CY_SCB_UART_SUCCESS)
    {
        HandleError();
    }
    
    /*Start the UART*/
    Cy_SCB_UART_Enable(UART_HW);
    
    Cy_SCB_UART_PutString(UART_HW, "EmEEPROM demo \n\r");
       
    /*Read 15 bytes out of EEPROM memory*/
    eepromReturnValue = Cy_Em_EEPROM_Read(LOGICAL_EEPROM_START, eepromArray, LOGICAL_EEPROM_SIZE, &Em_EEPROM_context);
    if(eepromReturnValue != CY_EM_EEPROM_SUCCESS)
    {
        HandleError();
    }
    
    /*If first byte of EEPROM isn't 'P' the write data from array*/
    if(eepromArray[0] != ASCII_P)
    {
        /*Write inital data to EEPROM*/
        eepromReturnValue = Cy_Em_EEPROM_Write(LOGICAL_EEPROM_START, array, LOGICAL_EEPROM_SIZE, &Em_EEPROM_context); 
        if(eepromReturnValue != CY_EM_EEPROM_SUCCESS)
        {
            HandleError();
        }
    }
    /*Otherwise it has been written update counter*/
    else
    {
        /*Read contents of EEPROM*/
        eepromReturnValue = Cy_Em_EEPROM_Read(LOGICAL_EEPROM_START, eepromArray, LOGICAL_EEPROM_SIZE, &Em_EEPROM_context);
        if(eepromReturnValue != CY_EM_EEPROM_SUCCESS)
        {
            HandleError();
        }
        
        /*Increment Counter by 1*/
        eepromArray[RESET_COUNT_LOCATION+1]++; 
        
        /*Counter is in ASCII, so handle overflow*/
        if(eepromArray[RESET_COUNT_LOCATION+1] > ASCII_NINE)
        { 
            /*Set lower digit to zero*/
            eepromArray[RESET_COUNT_LOCATION+1] = ASCII_ZERO;
            /*Increment upper digit*/
            eepromArray[RESET_COUNT_LOCATION]++;
            
            /*only increment to 99*/
            if(eepromArray[RESET_COUNT_LOCATION] > ASCII_NINE)
            {
                eepromArray[RESET_COUNT_LOCATION] = ASCII_NINE;
                eepromArray[RESET_COUNT_LOCATION+1] = ASCII_NINE;
            }
        }
        
        /*Only update the two count values in the EEPROM*/
        eepromReturnValue = Cy_Em_EEPROM_Write(RESET_COUNT_LOCATION, &eepromArray[RESET_COUNT_LOCATION], RESET_COUNT_SIZE, &Em_EEPROM_context);
        if(eepromReturnValue != CY_EM_EEPROM_SUCCESS)
        {
            HandleError();
        }
    }
    
    /*Read contents of EEPROM after write*/
    eepromReturnValue = Cy_Em_EEPROM_Read(LOGICAL_EEPROM_START, eepromArray, LOGICAL_EEPROM_SIZE, &Em_EEPROM_context);
    if(eepromReturnValue != CY_EM_EEPROM_SUCCESS)
    {
        HandleError();
    }
    
    /*Print EEPROM contents to console*/
    Cy_SCB_UART_PutArray(UART_HW, eepromArray, LOGICAL_EEPROM_SIZE); 
    Cy_SCB_UART_PutString(UART_HW, "\n\r");
    
    Cy_SCB_UART_PutString(UART_HW, "PRINTED FROM THE CM0 CORE \n\r");
    
 

    for(;;)
    {
        /* Place your application code here. */
    }
}

void HandleError(void)
{   
    Cy_SCB_UART_PutString(UART_HW, "Error! \n\r");
     /* Disable all interrupts. */
    __disable_irq();
    
    /* Infinite loop. */
    while(1u) {}
}
/* [] END OF FILE */
